import { Utilities } from '@/util/Utilities';
import {
    Button, Footer, Frame, Modal, ModalService, Select,
    Tabs,
    TextInput,
} from '@zeedhi/common';
import { Metadata } from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';

export class AgendamentoRecursosController {
    private visibilidadeGridAgendRecursos: boolean = true;

    protected currentRow : any = {};

    private modal!: Modal;

    public get visibilidadeFrameAgendRecursos() {
        return !this.visibilidadeGridAgendRecursos;
    }

    public get tituloFrame() : string {
        return this.currentRow.NRCONTRATOFCT;
    }

    public openFilter({ component }: any) {
        const filterButton = Metadata.getInstance<Button>(`${component.name}_filterButton`);
        filterButton.click({} as Event, {} as HTMLElement);
    }

    public async abrirModalAddAgendamentoRecurso() {
        this.modal = await ModalService.createFromJson('agendamentoRecursos/modal/modalAddAgendamentoRecurso', true);
        this.modal.show();
    }

    public fecharModalAddAgendRecursos() {
        this.modal.hide();
        this.modal?.destroy();
    }

    public async abrirmodalAddPessoa() {
        this.modal = await ModalService.createFromJson('agendamentoRecursos/modal/modalAddPessoa', true);
        this.modal.show();
    }

    public fecharModalAddPessoa() {
        this.modal.hide();
        this.modal?.destroy();
    }

    public abrirFrameAgendRecursos({ row }: any) {
        this.visibilidadeGridAgendRecursos = false;
        this.currentRow = row;
        this.loadTabsAgendamentoRecursos();
    }
    
    public voltarGridAgendRecursos() {
        this.visibilidadeGridAgendRecursos = true;
        this.ativaTabDescricao();
        this.reloadTabDescricao();
    }

    public ativaTabDescricao() {
        const setOfTabs = Metadata.getInstance<Tabs>('tabs-agendamento-recursos');
        setOfTabs.activeTab = 0;
    }

    public reloadTabDescricao() {
        const frame = Metadata.getInstance<Frame>('descricaoAgendamentoRecursos');
        frame.reload();
    }

    public setFilterFieldLocalAtendimento() {
        const localAtendimento = Metadata.getInstance<Select>('NMESTRUTURAM');

        localAtendimento.datasource.setFilter({
            NRSEQCONTRATO: null,
            TESTE: 'TESTE',
        });
    }

    public loadTabsAgendamentoRecursos() {
        const setOfTabs = Metadata.getInstance<Tabs>('tabs-agendamento-recursos');
        const footer = Metadata.getInstance<Footer>('footer-tab-descricao-agendamento-recursos');

        if (setOfTabs.tabs[setOfTabs.activeTab].name === 'tab_descricaoAgendamentoRecursos') {
            footer.isVisible = true;
            this.loadTabDescAgendamentoRecursos();
        } else if (setOfTabs.tabs[setOfTabs.activeTab].name === 'tab_equipeTrabalhoAgendamentoRecursos') {
            footer.isVisible = false;
            setTimeout(() => {
                this.loadTabEquipeTrabalhoAgendamentoRecursos();
            }, 1000);
        }
    }

    public loadTabDescAgendamentoRecursos() {
        Metadata.getInstance<Select>('FORM_TAB_DESCRICAO-NRSEQCONTRATO').setValue(this.currentRow.NRSEQCONTRATO);
    }

    public loadTabEquipeTrabalhoAgendamentoRecursos() {
        try {
            const grid = Metadata.getInstance<TekGrid>('tekTreeGrid-equipe-trabalho-agendamento-recursos');
            grid.datasource.setFilter({
                NRAGENDRECURSOCONT: this.currentRow.NRAGENDRECURSOCONT,
            });
        } catch (error) {
            Utilities.showErrorMessage(error);
        }
    }

    public setFieldsDescCont({ component }: any) {
        const nrseqcontrato = Metadata.getInstance<Select>(`${component.name.split('-')[0]}-NRSEQCONTRATO`).value;
        const nmsecundariparc = Metadata.getInstance<TextInput>(`${component.name.split('-')[0]}-NMSECUNDARIPARC`);
        const dsreferenciacont = Metadata.getInstance<TextInput>(`${component.name.split('-')[0]}-DSREFERENCIACONT`);

        component.datasource.data.forEach((dados: any) => {
            if (dados.NRSEQCONTRATO === nrseqcontrato || nrseqcontrato !== null) {
                nmsecundariparc.value = dados.NMSECUNDARIPARC;
                dsreferenciacont.value = dados.DSREFERENCIACONT;
            } else {
                nmsecundariparc.value = null;
                dsreferenciacont.value = null;
            }
        });
    }

    public saveNewAgendRecursos() {

    }

    public saveEditAgendRecursos() {

    }
}
