import { Utilities } from '@/util/Utilities';
import {
    DialogService, Footer, Form, LoadingService, Modal, ModalService, Select, SelectTree, SelectTreeMultiple, Tabs, TextInput, Textarea,
    TreeGrid,
} from '@zeedhi/common';
import { Http, IEventParam, Metadata } from '@zeedhi/core';
import { TekGrid, TekTreeGrid } from '@zeedhi/teknisa-components-common';
import { ZdSelectTree, ZdText } from '@zeedhi/vuetify';

export class ComposicaoServicosController {
    public visibilidade: boolean = true;
    
    private basicModal!: Modal;

    public currentRow: any = {};

    public dadosFiltroComposicao: any = {};

    get visibilidadeDoFrame() {
        return !this.visibilidade;
    }

    get textFrameComposicaoServicos() {
        return this.currentRow.NMCOMPSERVICO;
    }

    public async exibirModalFilterComposicao() {
        this.basicModal = await ModalService.createFromJson('/composicaoServicos/modal/composicaoServicosFilterModal', true);
        this.basicModal.show();
    }

    public async abrirModalFilterComposicaoServicos({ event }: IEventParam<TekGrid>) {
        event?.preventDefault();
        await this.exibirModalFilterComposicao();
        Metadata.getInstance<ZdText>('text-de-titulo-composicao-servicos').text = 'Filtro';
        
        setTimeout(() => {
            Metadata.getInstance<TextInput>('form-filter-composicao-servicos-NMCOMPSERVICO').value = this.dadosFiltroComposicao.NMCOMPSERVICO;
            Metadata.getInstance<Select>('form-filter-composicao-servicos-IDATIVO').value = 'S';
            Metadata.getInstance<Select>('form-filter-composicao-servicos-IDCOMPOBRIGATORIA').value = this.dadosFiltroComposicao.IDCOMPOBRIGATORIA;
            Metadata.getInstance<SelectTree>('form-filter-composicao-servicos-NRCOMPSERVICO-SUPERIOR').value = this.dadosFiltroComposicao.NRCOMPSERVICO;
            Metadata.getInstance<Textarea>('form-filter-composicao-servicos-DSCOMPSERVICO').value = this.dadosFiltroComposicao.FORMACAOPRECO;
            Metadata.getInstance<Select>('form-filter-composicao-servicos-CDCENTCUST').value = this.dadosFiltroComposicao.CDCENTCUST;
            Metadata.getInstance<Select>('form-filter-composicao-servicos-IDUTILIZAREVENDA').value = this.dadosFiltroComposicao.IDUTILIZAREVENDA;
        }, 200);
    }

    public setFilterComposicaoServicos() {
        const grid = Metadata.getInstance<TreeGrid>('tree-grid-composicao-servicos');
        grid.datasource.setFilter({
            IDATIVO: 'S',
        });
    }

    public fecharModalFilterComposicaoServicos() {
        this.basicModal.hide();
        this.basicModal?.destroy();
    }

    public filtrarComposicaoServicos() {
        const grid = Metadata.getInstance<TekTreeGrid>('tree-grid-composicao-servicos');
        const form = Metadata.getInstance<Form>('form-filter-composicao-servicos');

        this.dadosFiltroComposicao.NMCOMPSERVICO = Metadata.getInstance<TextInput>('form-filter-composicao-servicos-NMCOMPSERVICO').value;
        this.dadosFiltroComposicao.IDATIVO = Metadata.getInstance<Select>('form-filter-composicao-servicos-IDATIVO').value;
        this.dadosFiltroComposicao.IDCOMPOBRIGATORIA = Metadata.getInstance<Select>('form-filter-composicao-servicos-IDCOMPOBRIGATORIA').value;
        this.dadosFiltroComposicao.NRCOMPSERVICO = Metadata.getInstance<SelectTree>('form-filter-composicao-servicos-NRCOMPSERVICO-SUPERIOR').value;
        this.dadosFiltroComposicao.DSCOMPSERVICO = Metadata.getInstance<Textarea>('form-filter-composicao-servicos-DSCOMPSERVICO').value;
        this.dadosFiltroComposicao.CDCENTCUST = Metadata.getInstance<Select>('form-filter-composicao-servicos-CDCENTCUST').value;
        this.dadosFiltroComposicao.IDUTILIZAREVENDA = Metadata.getInstance<Select>('form-filter-composicao-servicos-IDUTILIZAREVENDA').value;

        if (Utilities.validateForm(form)) {
            grid.datasource.setFilter({
                NMCOMPSERVICO: Metadata.getInstance<TextInput>('form-filter-composicao-servicos-NMCOMPSERVICO').value,
                IDATIVO: Metadata.getInstance<Select>('form-filter-composicao-servicos-IDATIVO').value,
                IDCOMPOBRIGATORIA: Metadata.getInstance<Select>('form-filter-composicao-servicos-IDCOMPOBRIGATORIA').value,
                NRCOMPSERVICO: Metadata.getInstance<SelectTree>('form-filter-composicao-servicos-NRCOMPSERVICO-SUPERIOR').value,
                DSCOMPSERVICO: Metadata.getInstance<Textarea>('form-filter-composicao-servicos-DSCOMPSERVICO').value,
                CDCENTCUST: Metadata.getInstance<Select>('form-filter-composicao-servicos-CDCENTCUST').value,
                IDUTILIZAREVENDA: Metadata.getInstance<Select>('form-filter-composicao-servicos-IDUTILIZAREVENDA').value,
            });
            this.fecharModalFilterComposicaoServicos();
        }
    }

    public async abrirModalAddComposicaoServicos() {
        await this.exibirModalAddComposicaoServicos();
        Metadata.getInstance<ZdText>('text-de-titulo-composicao-servicos').text = 'Nova Composição de Serviços';
        this.capturaComposicaoSuperior();
    }

    public capturaComposicaoSuperior() {
        const grid = Metadata.getInstance<TekTreeGrid>('tree-grid-composicao-servicos');
        const composicaoSuperior = Metadata.getInstance<Select>('form-adicionar-composicao-servicos-NRCOMPSERVICO-SUPERIOR');
        setTimeout(() => {
            composicaoSuperior.value = grid.datasource.currentRow.NRCOMPSERVICOPAI;
        }, 10);
    }

    public async exibirModalAddComposicaoServicos() {
        this.basicModal = await ModalService.createFromJson('composicaoServicos/modal/composicaoServicosAddModal', true);
        this.basicModal.show();
    }

    public fecharModalAddComposicaoServicos() {
        this.basicModal.hide();
        this.basicModal?.destroy();
    }

    public vizualizarInformacoesComposicaoServicos({ row }: any) {
        this.visibilidade = false;
        this.currentRow = row;
        this.recarregarTabs();
    }

    public voltarPaginaInicial() {
        this.visibilidade = true;
    }

    public recarregarTabs() {
        const setOfTabs = Metadata.getInstance<Tabs>('tabs-composicao-servicos');
        const footer = Metadata.getInstance<Footer>('footer-info-composicao-servicos');

        if (setOfTabs.tabs[setOfTabs.activeTab].name === 'tab-info-composicao-servicos') {
            footer.isVisible = true;
            this.montarInformacoesComposicaoServicos();
        } else if (setOfTabs.tabs[setOfTabs.activeTab].name === 'tab-servicos') {
            footer.isVisible = false;
            setTimeout(() => {
                const grid = Metadata.getInstance<TekTreeGrid>('tree-grid-elemento-servicos');
                this.setFiltroGridElementoServico({ component: grid });
            }, 1000);
        }
    }

    public montarInformacoesComposicaoServicos() {
        LoadingService.show();
        Metadata.getInstance<TextInput>('form-info-composicao-servicos-NRCOMPSERVICO').value = this.currentRow.NRCOMPSERVICO;
        Metadata.getInstance<TextInput>('form-info-composicao-servicos-NMCOMPSERVICO').value = this.currentRow.NMCOMPSERVICO;
        Metadata.getInstance<Select>('form-info-composicao-servicos-IDATIVO').value = this.currentRow.IDATIVO;
        Metadata.getInstance<Select>('form-info-composicao-servicos-IDCOMPOBRIGATORIA').value = this.currentRow.IDCOMPOBRIGATORIA;
        Metadata.getInstance<SelectTree>('form-info-composicao-servicos-NRCOMPSERVICO-SUPERIOR').value = this.currentRow.NRCOMPSERVICOPAI;
        Metadata.getInstance<TextInput>('form-info-composicao-servicos-DSCOMPSERVICO').value = this.currentRow.DSCOMPSERVICO;
        Metadata.getInstance<Select>('form-info-composicao-servicos-CDCENTCUST').value = this.currentRow.CDCENTCUST;
        Metadata.getInstance<Select>('form-info-composicao-servicos-IDUTILIZAREVENDA').value = this.currentRow.IDUTILIZAREVENDA;
        LoadingService.hide();
    }

    // Fitro de Elemento de serviço para a rotina de cadastramento automático
    public setFiltroGridElementoServico({ component }: IEventParam<TekTreeGrid>) {
        component.datasource.setFilter({
            NRCOMPSERVICO: this.currentRow.NRCOMPSERVICO,
        });
    }

    public async abrirModalAddElementoServico() {
        await this.exibirModalAddElementoServico();
        Metadata.getInstance<ZdText>('text-de-titulo-elemento-servico').text = 'Novo Elemento de Serviço';
    }

    public async abrirModalEdicaoElementoServico({ row }: any) {
        await this.exibirModalAddElementoServico();

        const nmservorca = Metadata.getInstance<SelectTree>('form-elemento-servico-NMSERVORCA');
        const nmservorcapai = Metadata.getInstance<SelectTree>('form-elemento-servico-NMSERVORCAPAI');
        nmservorca.disabled = true;
        nmservorcapai.disabled = true;

        Metadata.getInstance<ZdText>('text-de-titulo-elemento-servico').text = 'Editar Elemento de Serviço';
        Metadata.getInstance<TextInput>('form-elemento-servico-NRITCOMPSERVICO').value = row.NRITCOMPSERVICO;
        Metadata.getInstance<TextInput>('form-elemento-servico-CDSERVORCA').value = row.CDSERVORCA;
        Metadata.getInstance<Select>('form-elemento-servico-IDSERVOBRIGCOMP').value = row.IDSERVOBRIGCOMP;
        Metadata.getInstance<TextInput>('form-elemento-servico-IDSERVOBRIGCOMP-BACKUP').value = row.IDSERVOBRIGCOMP;
        Metadata.getInstance<Select>('form-elemento-servico-IDAPARECEPROPOSTA').value = row.IDAPARECEPROPOSTA;
        setTimeout(() => {
            nmservorca.setValue(row.CDSERVORCA);
            nmservorcapai.setValue(row.IDELEMENTOPAI);
            nmservorca.disabled = false;
            nmservorcapai.disabled = false;
        }, 15000);
    }

    public async exibirModalAddElementoServico() {
        this.basicModal = await ModalService.createFromJson('composicaoServicos/modal/elementoServicoModal', true);
        this.basicModal.show();
    }

    public fecharModalElementoServico() {
        this.basicModal.hide();
        this.basicModal?.destroy();
    }

    public setFilterElementoSuperior({ component }: any) {
        const nrcompserv = Metadata.getInstance<TekTreeGrid>('tree-grid-composicao-servicos').datasource.currentRow.NRCOMPSERVICO;
        const nritcompservico = Metadata.getInstance<TekTreeGrid>('tree-grid-elemento-servicos').datasource.currentRow.NRITCOMPSERVICO;
        component.datasource.setFilter({
            NRCOMPSERVICO: nrcompserv,
            NRITCOMPSERVICO: nritcompservico,
        });
    }

    // Valida se o Elemento de Serviço Superior escolhido não é o mesmo Elemento que se está editando.
    public validarElementoServicoSuperior({ component }: any) {
        const idelemento = Metadata.getInstance<ZdSelectTree>('form-elemento-servico-NMSERVORCA').value;

        if (component.value) {
            if (component.value.CDSERVORCA === idelemento.CDSERVORCA) {
                Utilities.showDialogMessage('error', 'Operação bloqueada!', 'Um Elemento de Serviço não pode ser superior a ele mesmo.');
            }
        }
    }

    public async abrirModalCadastramentoAutomatico() {
        await this.exibirModalCadastramentoAutomatico();
        Metadata.getInstance<ZdText>('text-de-titulo-cadastramento-automatico').text = 'Cadastramento Automático';
    }

    public async exibirModalCadastramentoAutomatico() {
        this.basicModal = await ModalService.createFromJson('composicaoServicos/modal/cadastramentoAutomaticoModal', true);
        this.basicModal.show();
    }

    public fecharModalCadastramentoAutomatico() {
        this.basicModal.hide();
        this.basicModal?.destroy();
    }

    public saveComposicaoServicosAdd() {
        const formComposicaoServicos = Metadata.getInstance<Form>('form-adicionar-composicao-servicos');
        const grid = Metadata.getInstance<TekTreeGrid>('tree-grid-composicao-servicos');
        const params = {
            NRCOMPSERVICO: Metadata.getInstance<TextInput>('form-adicionar-composicao-servicos-NRCOMPSERVICO').value,
            NMCOMPSERVICO: Metadata.getInstance<TextInput>('form-adicionar-composicao-servicos-NMCOMPSERVICO').value,
            DSCOMPSERVICO: Metadata.getInstance<Textarea>('form-adicionar-composicao-servicos-DSCOMPSERVICO').value,
            IDATIVO: Metadata.getInstance<Select>('form-adicionar-composicao-servicos-IDATIVO').value,
            NRCOMPSERVICOPAI: Metadata.getInstance<SelectTree>('form-adicionar-composicao-servicos-NRCOMPSERVICO-SUPERIOR').value,
            IDCOMPOBRIGATORIA: Metadata.getInstance<Select>('form-adicionar-composicao-servicos-IDCOMPOBRIGATORIA').value,
            CDCENTCUST: Metadata.getInstance<Select>('form-adicionar-composicao-servicos-CDCENTCUST').value,
            IDUTILIZAREVENDA: Metadata.getInstance<Select>('form-adicionar-composicao-servicos-IDUTILIZAREVENDA').value,
        };

        if (Utilities.validateForm(formComposicaoServicos)) {
            LoadingService.show();
            Http.post('/saveComposicaoServicos', params).then(() => {
                this.fecharModalAddComposicaoServicos();
                grid.datasource.get();
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }

    public saveComposicaoServicosEdit() {
        const formComposicaoServicosedit = Metadata.getInstance<Form>('form-info-composicao-servicos');
        const grid = Metadata.getInstance<TekTreeGrid>('tree-grid-composicao-servicos');

        const params = {
            NRCOMPSERVICO: Metadata.getInstance<TextInput>('form-info-composicao-servicos-NRCOMPSERVICO').value,
            NMCOMPSERVICO: Metadata.getInstance<TextInput>('form-info-composicao-servicos-NMCOMPSERVICO').value,
            DSCOMPSERVICO: Metadata.getInstance<Textarea>('form-info-composicao-servicos-DSCOMPSERVICO').value,
            IDATIVO: Metadata.getInstance<Select>('form-info-composicao-servicos-IDATIVO').value,
            NRCOMPSERVICOPAI: Metadata.getInstance<SelectTree>('form-info-composicao-servicos-NRCOMPSERVICO-SUPERIOR').value,
            IDCOMPOBRIGATORIA: Metadata.getInstance<Select>('form-info-composicao-servicos-IDCOMPOBRIGATORIA').value,
            CDCENTCUST: Metadata.getInstance<Select>('form-info-composicao-servicos-CDCENTCUST').value,
            IDUTILIZAREVENDA: Metadata.getInstance<Select>('form-info-composicao-servicos-IDUTILIZAREVENDA').value,
        };

        if (Utilities.validateForm(formComposicaoServicosedit)) {
            LoadingService.show();
            Http.post('/saveComposicaoServicos', params).then(() => {
                this.voltarPaginaInicial();
                grid.datasource.get();
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }

    public showConfirmDeleteMessage({ event }: any) {
        event!.preventDefault();
        DialogService.show({
            name: 'buttons-dialog-operador',
            type: 'warning',
            title: 'Excluir Registro',
            text: 'Deseja realmente excluir todos os registros selecionados?',
            buttons: [
                {
                    name: 'NAO',
                    component: 'ZdButton',
                    label: 'Não',
                    outline: true,
                    events: {
                        click: () => {
                            DialogService.hide();
                        },
                    },
                },
                {
                    name: 'SIM',
                    component: 'ZdButton',
                    label: 'Sim',
                    events: {
                        click: () => {
                            this.deleteComposicaoEmCascata();
                            DialogService.hide();
                        },
                    },
                },
            ],
        });
    }

    public deleteComposicaoEmCascata() {
        const grid = Metadata.getInstance<TekTreeGrid>('tree-grid-composicao-servicos');
        LoadingService.show();
        Http.post('/deleteComposicaoEmCascata', grid.selectedRows)
            .then(() => {
                grid.datasource.get();
                Utilities.showSuccessMessage();
            }).catch((error) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
    }

    public saveElementoServico() {
        const formelementoServico = Metadata.getInstance<Form>('form-elemento-servico');
        const gridElementoServico = Metadata.getInstance<TekTreeGrid>('tree-grid-elemento-servicos');
        const gridComposicaServico = Metadata.getInstance<TekTreeGrid>('tree-grid-composicao-servicos');
        const servorca = Metadata.getInstance<SelectTree>('form-elemento-servico-NMSERVORCA');
        const idelementopai = Metadata.getInstance<SelectTree>('form-elemento-servico-NMSERVORCAPAI');
         
        const params = {
            NRITCOMPSERVICO: Metadata.getInstance<TextInput>('form-elemento-servico-NRITCOMPSERVICO').value,
            NRCOMPSERVICO: gridComposicaServico.datasource.currentRow.NRCOMPSERVICO,
            IDSERVOBRIGCOMP: Metadata.getInstance<Select>('form-elemento-servico-IDSERVOBRIGCOMP').value,
            IDSERVOBRIGCOMP_BACKUP: Metadata.getInstance<TextInput>('form-elemento-servico-IDSERVOBRIGCOMP-BACKUP').value,
            CDSERVORCA: servorca.value.CDSERVORCA,
            IDELEMENTO: servorca.value.IDELEMENTO,
            IDTIPOELEMENTO: servorca.value.IDTIPOELEMENTO,
            IDELEMENTOPAI_VALIDACAO: servorca.value.IDELEMENTOPAI,
            IDELEMENTOPAI: (idelementopai.value === null) ? null : idelementopai.value.IDELEMENTO,
            IDAPARECEPROPOSTA: Metadata.getInstance<Select>('form-elemento-servico-IDAPARECEPROPOSTA').value,
        };
        
        if (Utilities.validateForm(formelementoServico)) {
            LoadingService.show();
            Http.post('/saveElementoServico', params).then(() => {
                this.fecharModalElementoServico();
                gridElementoServico.datasource.get();
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }

    public showConfirmDeleteMessageItemElemento({ event }: any) {
        event!.preventDefault();
        DialogService.show({
            name: 'buttons-dialog-operador',
            type: 'warning',
            title: 'Excluir Registro',
            text: 'Deseja realmente excluir todos os registros selecionados?',
            buttons: [
                {
                    name: 'NAO',
                    component: 'ZdButton',
                    label: 'Não',
                    outline: true,
                    events: {
                        click: () => {
                            DialogService.hide();
                        },
                    },
                },
                {
                    name: 'SIM',
                    component: 'ZdButton',
                    label: 'Sim',
                    events: {
                        click: () => {
                            this.deleteItemComposicaoEmCascata();
                            DialogService.hide();
                        },
                    },
                },
            ],
        });
    }

    public deleteItemComposicaoEmCascata() {
        const grid = Metadata.getInstance<TekTreeGrid>('tree-grid-elemento-servicos');
        LoadingService.show();
        Http.post('/deleteItemComposicaoEmCascata', grid.selectedRows)
            .then(() => {
                grid.datasource.get();
                Utilities.showSuccessMessage();
            }).catch((error) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
    }

    public salvarCadastramentoAutomatico() {
        const formCadastramentoAutomatico = Metadata.getInstance<Form>('form-cadastramento-automatico');
        const gridElementoServico = Metadata.getInstance<TekTreeGrid>('tree-grid-elemento-servicos');
        const gridComposicaServico = Metadata.getInstance<TekTreeGrid>('tree-grid-composicao-servicos');
        const servorcaArray = Metadata.getInstance<SelectTreeMultiple>('form-cadastramento-automatico-NMSERVORCA').value;

        const params = {
            NRCOMPSERVICO: gridComposicaServico.datasource.currentRow.NRCOMPSERVICO,
            SERVORCA_ARRAY: servorcaArray,
            IDSERVOBRIGCOMP: Metadata.getInstance<Select>('form-cadastramento-automatico-IDSERVOBRIGCOMP').value,
            IDAPARECEPROPOSTA: Metadata.getInstance<Select>('form-cadastramento-automatico-IDAPARECEPROPOSTA').value,
        };
        
        if (Utilities.validateForm(formCadastramentoAutomatico)) {
            LoadingService.show();
            Http.post('/saveCadastramentoAutomatico', params).then(() => {
                this.fecharModalCadastramentoAutomatico();
                gridElementoServico.datasource.get();
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }

    public sincronizarComposicao() {
        const gridComposicaServico = Metadata.getInstance<TekTreeGrid>('tree-grid-composicao-servicos');

        const params = {
            NRCOMPSERVICO: gridComposicaServico.datasource.currentRow.NRCOMPSERVICO,
        };
        
        LoadingService.show();
        Http.post('/sincronizarComposicao', params).then(() => {
            Utilities.showSuccessMessage();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }
}
